"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const child_process_1 = require("child_process");
const promises_1 = require("fs/promises");
const temporarilyMovedFiles = {
    'package.json': `.package.json-${process.pid}`,
    'package-lock.json': `.package-lock.json-${process.pid}`,
};
class NPM {
    static async renamePackageJson() {
        for (const [name, replacement] of Object.entries(temporarilyMovedFiles)) {
            try {
                // Note: we don't need the return value.
                // lstat is only used to check that the file exists.
                await (0, promises_1.lstat)(name);
                await (0, promises_1.rename)(name, replacement);
            }
            catch (e) {
                console.debug(`no ${name} to archive`);
            }
        }
    }
    static async restorePackageJson() {
        for (const [name, replacement] of Object.entries(temporarilyMovedFiles)) {
            try {
                // Note: we don't need the return value.
                // lstat is only used to check that the file exists.
                await (0, promises_1.lstat)(replacement);
                await (0, promises_1.rename)(replacement, name);
            }
            catch (e) {
                console.debug(`no ${name} was archived`);
            }
        }
    }
    static async install(nodeCtx, pkgs) {
        await this.renamePackageJson();
        const p = (0, child_process_1.spawn)(nodeCtx.nodePath, [nodeCtx.npmPath, 'install', ...pkgs]);
        p.stdout.pipe(process.stdout);
        p.stderr.pipe(process.stderr);
        const exitPromise = new Promise((resolve) => {
            p.on('exit', (exitCode) => {
                resolve(exitCode);
            });
        });
        const exitCode = await exitPromise;
        await this.restorePackageJson();
        return exitCode;
    }
    static configure(nodeCtx, cfg) {
        return new Promise((resolve) => {
            const args = Object.keys(cfg)
                .filter((k) => cfg[k] !== null && cfg[k] !== undefined)
                .map((k) => `${k}=${cfg[k]}`);
            const p = (0, child_process_1.spawn)(nodeCtx.nodePath, [
                nodeCtx.npmPath,
                'config',
                'set',
                ...args,
            ]);
            p.stdout.pipe(process.stdout);
            p.stderr.pipe(process.stderr);
            p.on('exit', () => {
                console.log('Finished');
                resolve(0);
            });
        });
    }
    static rebuild(nodeCtx, ...args) {
        return new Promise((resolve) => {
            const p = (0, child_process_1.spawn)(nodeCtx.nodePath, [nodeCtx.npmPath, 'rebuild', ...args]);
            p.stdout.pipe(process.stdout);
            p.stderr.pipe(process.stderr);
            p.on('exit', (exitCode) => {
                resolve(exitCode);
            });
        });
    }
}
exports.default = NPM;
//# sourceMappingURL=npm.js.map